import networkx as nx
import enum


class ShortestPathAlgorithm(enum.Enum):
    """
    Enum class to present the different algorithms available
    """
    dijkstra = 1
    a_star = 2
    bellman_ford = 3


def nx_shortest_path(graph, src_entry_nodes, dst_entry_nodes, alg_name=ShortestPathAlgorithm.dijkstra):
    """
    Initialize process to find the shortest path in graph based on algorithm
    :param graph: NetworkX graph
        graph used for path finding
    :param src_entry_nodes: NetworkX node
        source node from graph
    :param dst_entry_nodes: NetworkX node
        destination node from graph
    :param alg_name: enum class ShortestPathAlgorithm attribute
        selected algorithm to determine path
    :return: A path between src and dst
    """
    try:
        if alg_name.value == 1:
            return __nx_dijkstra(graph, src_entry_nodes, dst_entry_nodes)
        elif alg_name.value == 2:
            return __nx_astar(graph, src_entry_nodes, dst_entry_nodes)
        elif alg_name.value == 3:
            return __nx_bellman_ford(graph, src_entry_nodes, dst_entry_nodes)
    except nx.NetworkXNoPath:
        return None


def __nx_dijkstra(graph, src_entry_nodes, dst_entry_nodes):
    """
    Find shortest path between src and dst using Dijkstra's algorithm
    :param graph: NetworkX graph
        graph used for path finding
    :param src_entry_nodes: list of NetworkX nodes
        source nodes from graph
    :param dst_entry_nodes: list of NetworkX nodes
        destination nodes from graph
    :return: list of NetworkX nodes describing the shortest path between src and dst
    """
    path = None
    path_length = None

    for src_entrance in src_entry_nodes:
        for dst_entrance in dst_entry_nodes:
            if path is None and path_length is None:

                path = nx.dijkstra_path(graph, source=src_entrance, target=dst_entrance)
                path_length = nx.dijkstra_path_length(graph, source=src_entrance, target=dst_entrance)
            else:
                temp_length = nx.dijkstra_path_length(graph, source=src_entrance, target=dst_entrance)
                if temp_length < path_length:
                    path = nx.dijkstra_path(graph, source=src_entrance, target=dst_entrance)
                    path_length = temp_length

    return path


def __nx_astar(graph, src_entry_nodes, dst_entry_nodes):
    """
    Find shortest path between src and dst using A star algorithm
    :param graph: NetworkX graph
        graph used for path finding
    :param src_entry_nodes: list of NetworkX nodes
        source nodes from graph
    :param dst_entry_nodes: list of NetworkX nodes
        destination nodes from graph
    :return: list of NetworkX nodes describing the shortest path between src and dst
    """
    path = None
    path_length = None

    for src_entrance in src_entry_nodes:
        for dst_entrance in dst_entry_nodes:
            if path is None and path_length is None:

                path = nx.astar_path(graph, source=src_entrance, target=dst_entrance)
                path_length = nx.astar_path_length(graph, source=src_entrance, target=dst_entrance)
            else:
                temp_length = nx.astar_path_length(graph, source=src_entrance, target=dst_entrance)
                if temp_length < path_length:
                    path = nx.astar_path(graph, source=src_entrance, target=dst_entrance)
                    path_length = temp_length

    return path


def __nx_bellman_ford(graph, src_entry_nodes, dst_entry_nodes):
    """
    Find shortest path between src and dst using Bellman-Ford's algorithm
    :param graph: NetworkX graph
        graph used for path finding
    :param src_entry_nodes: list of NetworkX nodes
        source nodes from graph
    :param dst_entry_nodes: list of NetworkX nodes
        destination nodes from graph
    :return: list of NetworkX nodes describing the shortest path between src and dst
    """
    path = None
    path_length = None

    for src_entrance in src_entry_nodes:
        for dst_entrance in dst_entry_nodes:
            if path is None and path_length is None:

                path = nx.bellman_ford_path(graph, source=src_entrance, target=dst_entrance)
                path_length = nx.bellman_ford_path_length(graph, source=src_entrance, target=dst_entrance)
            else:
                temp_length = nx.bellman_ford_path_length(graph, source=src_entrance, target=dst_entrance)
                if temp_length < path_length:
                    path = nx.bellman_ford_path(graph, source=src_entrance, target=dst_entrance)
                    path_length = temp_length

    return path




from shp_way.read_shp import ShapefileGraph
from shp_way.spatial_grid import SpatialGrid
from shapely.geometry import Point
from scipy import spatial


class CollisionDetection:
    """
    Class represents a collision detection object. Object determines entry points of all visitation objects using
    collision detection.

    Public Attributes
    ------------------
    - num_rows: number of rows used in spatial grid used for collision detection. Users can provide a set value or have
    value auto-generated. Value will be auto-generated by default.
    - num_cols: number of columns used in spatial grid used for collision detection. Users can provide a set value or
    have value auto-generated. Value will be auto-generated by default.

    'Private' Attributes
    ------------------
    - grid: SpatialGrid object
    - shp_graph: ShapefileGraph object
    """
    def __init__(self, read_obj, rows=None, cols=None):
        """
        Parameters
        ----------
        :param read_obj: ReadShapefiles
            a ReadShapefiles object containing the shapefiles
        :param rows: int, optional
            number of rows to be used for grid instead of auto-generating the value
        :param cols: int, optional
            number of rows to be used for grid instead of auto-generating the value
        """
        self.grid = None
        self.shp_graph = None

        if (isinstance(rows, int) and rows > 1) and (isinstance(cols, int) and cols > 1):
            self.__num_rows = rows
            self.__num_cols = cols
        else:
            self.__num_rows = None
            self.__num_cols = None

        self.__create_grid(read_obj)
        self.__scan()

    def num_rows(self):
        """
        :return: generated num of rows used for the spatial grid
        """
        return self.__num_rows

    def num_cols(self):
        """
        :return: generated num of columns used for the spatial grid
        """
        return self.__num_cols

    def __create_grid(self, read_obj):
        """
        Creates the spatial grid to partition the collision detection process
        :param read_obj: ReadShapefile object
        :return: None
        """
        self.shp_graph = ShapefileGraph(read_obj)

        self.grid = SpatialGrid(self.shp_graph.bb_max, self.shp_graph.bb_min,
                                self.shp_graph.polygon_heights, self.shp_graph.polygon_widths,
                                num_rows=self.__num_rows, num_cols=self.__num_cols)
        self.__num_rows = self.grid.num_rows
        self.__num_cols = self.grid.num_cols

    def __scan(self):
        """
        Initialize collision detection process
        :return: None
        """
        self.__scan_nodes_to_cells()
        self.__scan_visitation_objects()

    def __scan_nodes_to_cells(self):
        """
        Determine which cell partitions each node falls upon
        :return: None
        """
        bbox_max = self.grid.absolute_max
        bbox_min = self.grid.absolute_min

        for node in self.shp_graph.graph.nodes:
            if isinstance(node, tuple):
                x = node[0]
                y = node[1]

                col_cell = int((((x - bbox_min[0]) / (bbox_max[0] - bbox_min[0])) * self.__num_cols))
                row_cell = int((((y - bbox_min[1]) / (bbox_max[1] - bbox_min[1])) * self.__num_rows))
                self.grid.grid[row_cell][col_cell].append((float(node[0]), float(node[1])))

    def __scan_visitation_objects(self):
        """
        Initialize process to scanning for each visitation objects
        :return: None
        """
        for building, directory in self.shp_graph.reference_directory.items():
            self.__scan_visitation_objects_to_cells(directory)
            self.__assign_entry_points(directory)

    def __scan_visitation_objects_to_cells(self, directory):
        """
        Determine which cell partitions a visitation object falls upon using the objects' individual
        reference directory
        :param directory: hash table
            needed information relating each visitation object's sub-directory
        :return: None
        """
        polygon = directory['shp_reference']
        bbox_max = self.grid.absolute_max
        bbox_min = self.grid.absolute_min

        min_xy = (polygon.bounds[0], polygon.bounds[1])
        max_xy = (polygon.bounds[2], polygon.bounds[3])

        min_cell = (int((((min_xy[0] - bbox_min[0])/(bbox_max[0] - bbox_min[0])) * self.__num_cols)),
                    int((((min_xy[1] - bbox_min[1]) / (bbox_max[1] - bbox_min[1])) * self.__num_rows)))

        max_cell = (int((((max_xy[0] - bbox_min[0])/(bbox_max[0] - bbox_min[0])) * self.__num_cols)),
                    int((((max_xy[1] - bbox_min[1]) / (bbox_max[1] - bbox_min[1])) * self.__num_rows)))

        for i in range(min_cell[0], max_cell[0] + 1, 1):
            for j in range(min_cell[1], max_cell[1] + 1, 1):
                directory['assigned_cell_partitions'].append((j, i))

    def __assign_entry_points(self, directory):
        """
        Assign entry points for all visitation objects using references of nodes
        :param directory: hash table
            needed information relating each visitation object's sub-directory
        :return: None
        """
        polygon = directory['shp_reference']
        count = 0

        for cell in directory['assigned_cell_partitions']:
            x = cell[0]
            y = cell[1]

            if len(self.grid.grid[x][y]) != 0:
                for node in self.grid.grid[x][y]:
                    node_pt = Point((float(node[0]), float(node[1])))

                    if node_pt.intersects(polygon) and (float(node[0]), float(node[1])) not in \
                            directory['entry_nodes']:
                        count += 1
                        directory['entry_nodes'].append((float(node[0]), float(node[1])))

        if count == 0:
            found_nodes = list()

            mx = max(directory['assigned_cell_partitions'])
            mx_x = mx[0]
            mx_y = mx[1]

            mn = min(directory['assigned_cell_partitions'])
            mn_x = mn[0]
            mn_y = mn[1]

            node = self.shp_graph.midpoint(polygon.bounds[0], polygon.bounds[1], polygon.bounds[2], polygon.bounds[3])

            while len(found_nodes) == 0 and mx_x <= len(self.grid.grid) and mx_y < len(self.grid.grid) \
                    and mn_y >= 0 and mx_x >= 0:

                for r in range(mn_y, mx_y, 1):
                    for c in range(mn_x, mx_x, 1):
                        if len(self.grid.grid[c][r]) != 0:
                            found_nodes.extend(self.grid.grid[c][r])

                if mx_x < len(self.grid.grid):
                    mx_x += 1
                if mx_y < len(self.grid.grid):
                    mx_y += 1
                if mn_x > 0:
                    mn_x -= 1
                if mn_y > 0:
                    mn_y -= 1

            if len(found_nodes) == 0:
                found_nodes = self.shp_graph.graph.nodes

            tree = spatial.KDTree(found_nodes)
            tree.query([node])
            output = tree.query([node])
            directory['entry_nodes'].append(list(found_nodes).__getitem__(output[1][0]))



import shapefile
import networkx as nx
from shapely.geometry import Polygon, box
import math


"""Shapefile Represented Types"""
NULL = 0
POINT = 1
POLYLINE = 3
POLYGON = 5
MULTIPOINT = 8
POINTZ = 11
POLYLINEZ = 13
POLYGONZ = 15
MULTIPOINTZ = 18
POINTM = 21
POLYLINEM = 23
POLYGONM = 25
MULTIPOINTM = 28
MULTIPATCH = 31


class ShapeFileNavigatorException(Exception):
    pass


class ReadShapeFiles:
    """
    Class represents an object to contain two PyShp objects while ensuring shapefiles are supported

     Public Attributes
    ------------------
    - pathways_sf: PyShp object containing pathways shapefile
    - destinations_sf: PyShp object containing the destinations shapefile
    """
    def __init__(self, pathways="shapefiles/roads", destinations="shapefiles/buildings"):
        """
        Parameters
        ----------
        :param pathways: string, optional if shapefiles are present in 'shapefiles/' code directory
            directory containing shapefile
            example: 'C:/Username/documents/shapefile_folder/roads'

        :param destinations: destinations: string, optional if shapefiles are present in 'shapefiles/' code directory
            directory containing shapefile
            example: 'C:/Username/documents/shapefile_folder/buildings'
        """
        self.__pathways_sf = None
        self.__destinations_sf = None
        self.__integrity_check(pathways, destinations)

    @property
    def pathways_sf(self):
        """
        :return: shapefile containing pathways
        """
        return self.__pathways_sf

    @property
    def destinations_sf(self):
        """
        :return: shapefile containing visitation objects
        """
        return self.__destinations_sf

    def __integrity_check(self, pathways, destinations):
        """
        Ensure given shapefiles are supported and are aligned
        :param pathways: string
            Shapefile containing polyline shapes
        :param destinations: string
            Shapefile containing polygon shapes (Currently)
        :return: None
        """
        pathways_sf = self.__read_files(pathways)
        destinations_sf = self.__read_files(destinations)

        p_bbox = box(pathways_sf.bbox[0], pathways_sf.bbox[1], pathways_sf.bbox[2], pathways_sf.bbox[3])
        v_bbox = box(destinations_sf.bbox[0], destinations_sf.bbox[1], destinations_sf.bbox[2], destinations_sf.bbox[3])

        if pathways_sf.shapeType == POLYLINE and destinations_sf.shapeType == POLYGON:
            if p_bbox.intersects(v_bbox):
                self.__pathways_sf = pathways_sf
                self.__destinations_sf = destinations_sf
            else:
                raise ShapeFileNavigatorException("Scope of included shapefiles are not aligned")
        elif pathways_sf.shapeType == POLYGON and destinations_sf.shapeType == POLYLINE:
            if v_bbox.intersects(p_bbox):
                self.__pathways_sf = destinations_sf
                self.__destinations_sf = pathways_sf
            else:
                raise ShapeFileNavigatorException("Scope of included shapefiles are not aligned")
        else:
            raise ShapeFileNavigatorException("The included shapefiles are not supported")

    @staticmethod
    def __read_files(file_path):
        """
        Apply shapefile reader to main file
        :param file_path: string
            main file selected
        :return: PyShp reader of given file_path
        """
        sf = shapefile.Reader(file_path)
        return sf


class ShapefileGraph:
    """
    Class represents a graph and reference directory generated from shapefiles using a ReadShapefile object

    Public Attributes
    ------------------
    - bb_max: maximum bounding boxes vertex value from all visitation objects' polygon's
    - bb_min: minimum bounding boxes vertex value from all visitation objects' polygon's
    - polygon_widths: all visitation objects' polygon's bounding box's width
    - polygon_heights: all visitation objects' polygon's bounding box's height
    - graph: NetworkX graph object
    - reference directory: hash table data structure used to reference all visitation objects' polygon, cell partitions,
    and entry points
    """
    def __init__(self, readshp_obj):
        """
        Parameters
        ----------
        :param readshp_obj: ReadShapefiles object
            a ReadShapefiles object containing the shapefiles
        """
        self.__bb_max = None
        self.__bb_min = None

        self.__polygon_widths = list()
        self.__polygon_heights = list()

        self.__graph = nx.Graph()
        self.__reference_directory = dict()
        self.__process(readshp_obj)

    @property
    def bb_max(self):
        return self.__bb_max

    @property
    def bb_min(self):
        return self.__bb_min

    @property
    def polygon_widths(self):
        return self.__polygon_widths

    @property
    def polygon_heights(self):
        return self.__polygon_heights

    @property
    def graph(self):
        return self.__graph

    @property
    def reference_directory(self):
        return self.__reference_directory

    def __process(self, read_shp_obj):
        """
        Initialize the process to create the shapefile graph and reference directory
        :param read_shp_obj: ReadShapefile object
        :return: None
        """
        if not isinstance(read_shp_obj, ReadShapeFiles):
            raise ShapeFileNavigatorException("Given object is not supported. Please use a ReadShapefiles object")

        self.__process_polygons(read_shp_obj.destinations_sf)
        self.__process_poly_lines(read_shp_obj.pathways_sf)

        min_x = min(read_shp_obj.pathways_sf.bbox[0], read_shp_obj.destinations_sf.bbox[0])
        min_y = min(read_shp_obj.pathways_sf.bbox[1], read_shp_obj.destinations_sf.bbox[1])

        max_x = max(read_shp_obj.pathways_sf.bbox[2], read_shp_obj.destinations_sf.bbox[2])
        max_y = max(read_shp_obj.pathways_sf.bbox[3], read_shp_obj.destinations_sf.bbox[3])

        self.__bb_max = (max_x, max_y)
        self.__bb_min = (min_x, min_y)

    def __process_polygons(self, sf):
        """
        Process shapefiles containing polygons
        :param sf: PyShp object
            PyShp object containing the polygon shapefile
        :return: None
        """
        records = list(sf.iterRecords())
        counter = 0

        for i, shape in enumerate(sf.iterShapeRecords()):
            self.__calculate_polygon_size(shape.shape.bbox)
            name = records[i][1]

            if name.strip() is '':
                counter += 1
                name = "Unknown {}".format(counter)

            self.__reference_directory[name.strip()] = {
                'assigned_cell_partitions': list(),
                'shp_reference':  Polygon(shape.shape.points),
                'entry_nodes': list()
            }

    def __calculate_polygon_size(self, bbox):
        """
        Calculate the width and height of a polygon's bounding box
        :param bbox: list of floats
            bounding box of a polygon
        :return: None
        """
        mn_x = bbox[0]
        mn_y = bbox[1]
        mx_x = bbox[2]
        mx_y = bbox[3]

        mn = (mn_x, mn_y)
        mn_x_mx_y = (mx_x, mn_y)
        mx = (mx_x, mx_y)

        width = self.distance_calculation(mn, mn_x_mx_y)
        height = self.distance_calculation(mn_x_mx_y, mx)

        self.__polygon_widths.append(width)
        self.__polygon_heights.append(height)

    def __process_poly_lines(self, sf):
        """
        Process shapefiles containing polylines
        :param sf: PyShp object
            PyShp object containing the polyline shapefile
        :return: None
        """
        for shape in sf.iterShapeRecords():
            prev_point = None
            counter = 0

            for point in shape.shape.points:
                self.__graph.add_node(point, pos=point)
                if counter > 0:
                    self.__graph.add_edge(prev_point, point, weight=self.distance_calculation(prev_point, point))
                prev_point = point
                counter += 1

    @staticmethod
    def midpoint(x1, y1, x2, y2):
        """
        Calculate the midpoint using x and y values
        :param x1: float
            first x value
        :param y1: float
            first y value
        :param x2: float
            second x value
        :param y2: float
            second y value
        :return: midpoint
        """
        coord = ((x1 + x2)/2, (y1 + y2)/2)
        return coord

    @staticmethod
    def distance_calculation(coord1, coord2):
        """
        Calculate the distance between two coordinates
        :param coord1: tuple of floats
            first coordinate
        :param coord2: tuple of floats
            second coordinate
        :return: distance
        """
        x1 = coord1[0]
        y1 = coord1[1]
        x2 = coord2[0]
        y2 = coord2[1]
        return math.hypot(x2 - x1, y2 - y1)



import math
from statistics import median


class SpatialGrid:
    """"
    Class represents a spatial grid based on inputted attributes

    Public Attributes
    ------------------
    - grid: data structure representing the spatial grid
    - absolute_max: upper left coordinate of the grid
    - absolute_min: lower right coordinate of the grid
    - num_rows: number of rows presented in the grid
    - num_cols: number of columns presented in the grid

    'Private' Attributes
    ------------------
    - absolute_width: the medium value from a list of widths used for auto-generating number of columns in grid
    - absolute_height: the medium value from a list of heights used for auto-generating number of rows in grid
    - x_max: the max x coordinate
    - y_max: the max y coordinate
    - x_min: the min x coordinate
    - y_min: the min y coordinate

    """
    def __init__(self, abs_max, abs_min, heights, widths, num_rows=None, num_cols=None):
        """
        Parameters
        ----------
        :param abs_max : float
            absolute max coordinate for grid
        :param abs_min: float
            absolute min coordinate for grid
        :param heights: list (int)
            list of height values to approximate median cell size
        :param widths: list (int)
            list of width values to approximate median cell size
        :param num_rows: int, optional
            number of rows to be used for grid instead of auto-generating the value
        :param num_cols: int, optional
            number of rows to be used for grid instead of auto-generating the value
        """
        self.__absolute_max = None
        self.__absolute_min = None

        self.__absolute_height = None
        self.__absolute_width = None

        self.__grid = list()

        self.__grid_padding(abs_max, abs_min)

        self.__x_max = self.__absolute_max[0]
        self.__y_max = self.__absolute_max[1]

        self.__x_min = self.__absolute_min[0]
        self.__y_min = self.__absolute_min[1]

        if (isinstance(num_rows, int) and num_rows > 1) and (isinstance(num_cols, int) and num_cols > 1):
            self.__num_rows = num_rows
            self.__num_cols = num_cols
        else:
            self.__calculate_num_rows_cols(median(widths), median(heights))
        self.__create_spatial_grid(self.__num_rows, self.__num_cols)

    @property
    def grid(self):
        return self.__grid

    @property
    def absolute_max(self):
        return self.__absolute_max

    @property
    def absolute_min(self):
        return self.__absolute_min

    @property
    def num_rows(self):
        return self.__num_rows

    @property
    def num_cols(self):
        return self.__num_cols

    def __grid_padding(self, abs_max, abs_min):
        """
        Slightly pad grid boundaries to ensure all objects are covered by the spatial grid
        :param abs_max: tuple of floats
            the upper left vertex of the grid
        :param abs_min: tuple of floats
            the lower right vertex of the grid
        :return: None
        """
        abs_min = min(abs_max, abs_min)
        abs_max = max(abs_max, abs_min)

        x_max_temp = abs_max[0] + .00000001
        y_max_temp = abs_max[1] + .00000001
        x_min_temp = abs_min[0] + .00000001
        y_min_temp = abs_min[1] + .00000001

        self.__absolute_max = (x_max_temp, y_max_temp)
        self.__absolute_min = (x_min_temp, y_min_temp)

    @staticmethod
    def distance_calculation(coord1, coord2):
        """
        Calculate the distance between two coordinates
        :param coord1: tuple of floats
            first coordinate
        :param coord2: tuple of floats
            second coordinate
        :return: distance
        """
        x1 = coord1[0]
        y1 = coord1[1]
        x2 = coord2[0]
        y2 = coord2[1]
        return math.hypot(x2 - x1, y2 - y1)

    def __calculate_abs_len_width(self):
        """
        Calculate the median size of the width and height from the widths and heights parameters
        :return: None
        """
        self.__absolute_width = self.distance_calculation(self.__absolute_min, (self.__x_max, self.__y_min))
        self.__absolute_height = self.distance_calculation((self.__x_max, self.__y_min), self.__absolute_max)

    def __calculate_num_rows_cols(self, median_width, median_height):
        """
        Determine the number of rows and cols by finding the fixed size of each cell in the grid
        :param median_width: list of floats
        :param median_height: list of floats
        :return: None
        """
        self.__calculate_abs_len_width()

        self.__num_rows = int(self.__absolute_height / median_height)
        self.__num_cols = int(self.__absolute_width / median_width)

    def __create_spatial_grid(self, num_rows, num_cols):
        """
        Create a grid based on a given value for the number of rows and columns
        :param num_rows: int
            number of rows
        :param num_cols: int
            number of columns
        :return: None
        """
        self.__grid = list()

        for row in range(num_rows):
            temp_row = list()
            for col in range(num_cols):
                temp_col = list()
                temp_row.append(temp_col)
            self.__grid.append(temp_row)



from shp_way import collision_detection, path_finder, read_shp
import matplotlib.pyplot as plt
import mplleaflet
import random


class ShapefileNavigator:
    """
    Class represents a tool that converts shapefiles to a working navigation system. Size of spatial grid used for the
    system can be set to a fixed size or auto-generated.

    'Private' Attributes
    ------------------
    - r_obj: ReadShapeFiles object
    - collision_obj: CollisionDetection object


    Example of class usage:
    ShapefileNavigator('C:/Username/documents/shapefile_folder/roads',
                       'C:/Username/documents/shapefile_folder/buildings')
        - size of grid will be auto-generated due to not providing a value

    ShapefileNavigator('C:/Username/documents/shapefile_folder/roads',
                       'C:/Username/documents/shapefile_folder/buildings', rows=15, cols=20)

        - setting values for the rows and cols will create a fixed size set of cell partitions in the grid
        - 15 rows and 20 cols creates 300 cell partitions used for the spatial grid. Size of spatial grid can affect
        efficiency and processing speeds
    """
    def __init__(self, pathway_shapefile, visitation_shapefile, rows=None, cols=None):
        """
        Parameters
        ----------
        :param pathway_shapefile: string
            shapefile describing the pathways
        :param visitation_shapefile: string
            shapefile describes all potential visitation objects or destinations
        :param rows: int, optional
             number of rows to be used for grid instead of auto-generating the value
        :param cols: int, optional
             number of columns to be used for grid instead of auto-generating the value
        """

        self.__r_obj = read_shp.ReadShapeFiles(pathways=pathway_shapefile, destinations=visitation_shapefile)
        self.__collision_obj = collision_detection.CollisionDetection(self.__r_obj, rows=rows, cols=cols)

    def get_rows_cols(self):
        """
        Get the rows and columns used for the spatial grid for the shapefile navigation system
        :return: tuple of the number of rows and columns
        """

        return self.__collision_obj.num_rows(), self.__collision_obj.num_cols()

    def get_graph(self):
        """
        Get the shapefile graph used for the navigation system
        :return: NetworkX graph
        """

        return self.__collision_obj.shp_graph.graph

    def get_reference_directory(self):
        """
        Get the reference directory for all visitation objects
        :return: hash table containing all information regarding each visitation object
        """

        return self.__collision_obj.shp_graph.reference_directory

    def show_directory(self):
        """
        Print all potential areas of visitation
        :return: None
        """

        directory = self.get_reference_directory()
        for i in directory.keys():
            print(i)

    @staticmethod
    def get_algorithms():
        """
        Get all available algorithms
        :return: enum class containing all available algorithms
        """

        return path_finder.ShortestPathAlgorithm

    def find_path(self, src, dst, algorithm=path_finder.ShortestPathAlgorithm.dijkstra):
        """
        Calculate the path between src and dst using a supported path finding algorithm
        :param src: string
            the source or starting point for navigation
        :param dst: string
            the destination or ending point for navigation
        :param algorithm: ShortestPathAlgorithm attribute, optional (defaulted to Dijkstra's)
            algorithm used for path finding
        :return: list of NetworkX nodes outlining shortest path between src and dst
        """

        directory = self.get_reference_directory()
        if directory.get(src) is None or directory.get(dst) is None:
            raise read_shp.ShapeFileNavigatorException("src or dst is not present in reference directory")
        path = path_finder.nx_shortest_path(self.get_graph(), directory[src]['entry_nodes'],
                                            directory[dst]['entry_nodes'], alg_name=algorithm)
        return path

    def show_path(self, src, dst, algorithm=path_finder.ShortestPathAlgorithm.dijkstra,
                  show_graph=False, show_entry_points=True):
        """
        Calculate and display the path between src and dst using a supported path finding algorithm.
        :param src: string
            the source or starting point for navigation
        :param dst: string
            the destination or ending point for navigation
        :param algorithm: ShortestPathAlgorithm attribute, optional (defaulted to Dijkstra's)
            algorithm used for path finding
        :param show_graph: boolean, optional
            outline the NetworkX graph in which path finding takes place on
        :param show_entry_points: boolean, optional
            outline all potential entry points between src and dst
        :return: None
        """

        path = self.find_path(src, dst, algorithm=algorithm)
        if path is None:
            raise read_shp.ShapeFileNavigatorException("path does not exist")

        if show_graph:
            color = 'blue'
            line_width = 5
        else:
            color = (random.uniform(0, .5), random.uniform(0, .5), random.uniform(0, .5))
            line_width = 2

        x = list()
        y = list()
        for i in path:
            x.append(i[0])
            y.append(i[1])

        plt.plot(x, y, color=color, linestyle='solid', marker='o', linewidth=line_width)

        if show_graph is True:
            graph = self.get_graph()
            pos = path_finder.nx.get_node_attributes(graph, 'pos')
            path_finder.nx.draw(graph, pos, node_size=10)
            path_finder.nx.draw_networkx_labels(graph, pos, font_size=5, font_family='sans-serif')

        if show_entry_points is True:
            x_coordinates = [path[0][0], path[-1][0]]
            y_coordinates = [path[0][1], path[-1][1]]
            plt.scatter(x=x_coordinates, y=y_coordinates, color=color, marker='D')

        mplleaflet.show()



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name='shp_way',
    version='1.0',
    packages=setuptools.find_packages(),
    license='Apache',
    long_description=open('README.md').read(),
    url='https://github.com/bbrownrichardson/ShpWay',
    author='Brianna Brown Richardson',
    author_email='bbrownrichardson@gmail.com',
    install_requires=['pyshp', 'shapely', 'matplotlib', 'networkx', 'scipy', 'mplleaflet'],
    )
